#ifndef _KEYPADBUTTON_H_
#define _KEYPADBUTTON_H_

#define LGFX_AUTODETECT
#include <LovyanGFX.hpp>

class Keypad;

typedef void (*BUTTON_ON_PRESSED_CALLBACK)(void *, int);

class KeypadButton
{
public:
    KeypadButton(String label, int value, bool enabled, BUTTON_ON_PRESSED_CALLBACK on_pressed) : label{label}, value{value}, enabled{enabled}, on_pressed{on_pressed} {}
    KeypadButton(String label, int value, bool enabled, float x, float y, BUTTON_ON_PRESSED_CALLBACK on_pressed) : label{label}, value{value}, enabled{enabled}, x{x}, y{y}, on_pressed{on_pressed} {}
    KeypadButton(String label, int value, bool enabled, float x, float y, int width, int height, BUTTON_ON_PRESSED_CALLBACK on_pressed) : label{label}, value{value}, enabled{enabled}, x{x}, y{y}, width{width}, height{height}, on_pressed{on_pressed} {}

    void draw(LGFX *canvas)
    {
        if (!dirty)
        {
            return;
        }
        dirty = false;

        float MARGIN = 10.0;
        float HALF_MARGIN = MARGIN / 2.0;

        float new_x = x + HALF_MARGIN;
        float new_y = y + HALF_MARGIN;

        float new_width = width - MARGIN;
        float new_height = height - MARGIN;

        int bg_color = enabled ? TFT_WHITE : TFT_DARKGRAY;
        int fg_color = enabled ? TFT_BLACK : TFT_WHITE;

        canvas->setEpdMode(epd_fastest);
        canvas->setColor(bg_color);
        canvas->fillRect(new_x + 1, new_y + 1, new_width - 2, new_height - 2);

        canvas->setTextSize(4.0);
        canvas->setBaseColor(bg_color);
        canvas->setTextColor(fg_color);
        canvas->setCursor(x + (width / 2) - (canvas->fontWidth() / 2), y + (height / 2) - (canvas->fontHeight() / 2));
        canvas->print(label);

        canvas->setColor(TFT_BLACK);
        canvas->drawFastHLine(new_x, new_y, new_width);
        canvas->drawFastHLine(new_x, new_y + new_height, new_width);
        canvas->drawFastVLine(new_x, new_y, new_height);
        canvas->drawFastVLine(new_x + new_width, new_y, new_height);
    }

    ~KeypadButton() = default;

    String get_label()
    {
        return label;
    }
    void set_label(String value)
    {
        label = value;
    }

    int get_value()
    {
        return value;
    }
    void set_value(int value)
    {
        this->value = value;
    }

    int get_width()
    {
        return width;
    }
    void set_width(int value)
    {
        width = value;
    }

    int get_x()
    {
        return x;
    }
    void set_x(int value)
    {
        x = value;
    }

    int get_y()
    {
        return y;
    }
    void set_y(int value)
    {
        y = value;
    }

    void disable()
    {
        if (!enabled)
            return;

        enabled = false;
        dirty = true;
    }
    void enable()
    {
        if (enabled)
            return;

        enabled = true;
        dirty = true;
    }

    void update(Keypad *keypad, uint16_t touch_x, uint16_t touch_y)
    {
        if (enabled && touch_x > x && touch_x < x + width && touch_y > y && touch_y < y + height)
        {
            on_pressed(keypad, value);
        }
    }

private:
    String label;
    BUTTON_ON_PRESSED_CALLBACK on_pressed = NULL;
    bool enabled = true;
    bool dirty = true;

    int value = 0;

    int width = 0;
    int height = 0;

    float x = 0;
    float y = 0;
};

#endif