#ifndef _INPUT_H_
#define _INPUT_H_

#define LGFX_AUTODETECT
#include <LovyanGFX.hpp>

class Input
{
public:
    Input(String value) : value{value}, old_value{value} {}
    Input(String value, float x, float y) : value{value}, x{x}, y{y} {}
    Input(String value, float x, float y, int width, int height) : value{value}, x{x}, y{y}, width{width}, height{height} {}

    ~Input() = default;

    String get_value()
    {
        return value;
    }
    void set_value(String value)
    {
        this->value = value;
        dirty = true;
    }

    void append(String value)
    {
        this->value += value;
        dirty = true;
    }

    void append(int value)
    {
        this->append(String(value));
    }

    void clear()
    {
        this->value = "";
        dirty = true;
    }

    inline unsigned int length()
    {
        return this->value.length();
    }

    int get_width()
    {
        return width;
    }
    void set_width(int value)
    {
        width = value;
    }

    int get_x()
    {
        return x;
    }
    void set_x(int value)
    {
        x = value;
    }

    int get_y()
    {
        return y;
    }
    void set_y(int value)
    {
        y = value;
    }

    void draw(LGFX *canvas)
    {
        if (!dirty)
        {
            return;
        }
        dirty = false;

        float MARGIN = 10.0;
        float HALF_MARGIN = MARGIN / 2.0;

        float new_x = x + HALF_MARGIN;
        float new_y = y + HALF_MARGIN;

        float new_width = width - MARGIN;
        float new_height = height - MARGIN;

        int bg_color = TFT_WHITE;
        int fg_color = TFT_BLACK;

        canvas->setTextSize(8.0);

        canvas->setEpdMode(epd_fastest);
        canvas->setColor(bg_color);
        canvas->fillRect(new_x + 1, new_y + 1, new_width - 2, new_height - 2);

        canvas->setBaseColor(bg_color);
        canvas->setTextColor(fg_color);
        canvas->setCursor(x + (width / 2) - ((canvas->fontWidth() * value.length()) / 2), y + (height / 2) - (canvas->fontHeight() / 2));
        canvas->setEpdMode(epd_text);
        canvas->print(value);

        if (length() == 0)
        {
            canvas->setEpdMode(epd_fastest);
            canvas->setColor(TFT_BLACK);
            canvas->drawFastHLine(new_x, new_y, new_width);
            canvas->drawFastHLine(new_x, new_y + new_height, new_width);
            canvas->drawFastVLine(new_x, new_y, new_height);
            canvas->drawFastVLine(new_x + new_width, new_y, new_height);
        }
    }

private:
    LGFX *canvas;
    String value;
    String old_value;

    bool dirty = true;

    int width = 0;
    int height = 0;

    float x = 0;
    float y = 0;
};

#endif