#ifndef _KEYPAD_H_
#define _KEYPAD_H_

#include <EventEmitter.h>
#include "button.hpp"

constexpr int BUTTON_ROWS = 4;
constexpr int BUTTON_COLS = 3;
constexpr int BUTTON_COUNT = BUTTON_ROWS * BUTTON_COLS;

class Keypad : public EventEmitter<int>
{
public:
    Keypad(LGFX *canvas) : EventEmitter<int>(), canvas{canvas}
    {
        int BUTTON_SIZE = canvas->height() / BUTTON_ROWS;
        width = BUTTON_SIZE * BUTTON_COLS;

        for (int i = 0; i < BUTTON_COUNT; i++)
        {
            float x = BUTTON_SIZE * (i % BUTTON_COLS);
            float y = BUTTON_SIZE * (i / BUTTON_COLS);

            String lbl = String(i + 1);
            bool enabled = true;

            if (i == 9)
            {
                lbl = String("<");
            }
            if (i == 10)
            {
                lbl = String("0");
            }
            if (i == 11)
            {
                lbl = String(">");
                enabled = false;
            }

            buttons.push_back(new KeypadButton(lbl, i, enabled, x, y, BUTTON_SIZE, BUTTON_SIZE, Keypad::on_button_pressed));
        }
    }

    ~Keypad() = default;

    inline unsigned int get_width()
    {
        return width;
    }

    void update(tp_finger_t touch)
    {
        for (int i = 0; i < BUTTON_COUNT; i++)
        {
            buttons[i]->update(this, touch.x, touch.y);
        }
    }

    void draw()
    {
        for (int i = 0; i < BUTTON_COUNT; i++)
        {
            buttons[i]->draw(canvas);
        }
    }

    void disable(int index)
    {
        if (index >= 0 && index < BUTTON_COUNT)
        {
            buttons[index]->disable();
        }
    }

    void set_enabled(int index, bool enabled)
    {
        if (index >= 0 && index < BUTTON_COUNT)
        {
            if (enabled)
            {
                buttons[index]->enable();
            }
            else
            {
                buttons[index]->disable();
            }
        }
    }

private:
    LGFX *canvas;
    std::vector<KeypadButton *> buttons;
    bool touching = false;
    unsigned int width = 0;

    static void on_button_pressed(void *context, int value)
    {
        auto c = static_cast<Keypad *>(context);
        c->emit("pressed", c->buttons[value]->get_value());
    }
};

#endif